#!/bin/bash

## This Script work with Debian/Ubuntu
## It install Nginx by compile the source code
## All dependency will be installed before nginx.

## Version 0.4

## Do not ask options
export DEBIAN_FRONTEND=noninteractive
export DEBIAN_PRIORITY=critical

## Update debian repository
apt-get update > /dev/null 2>&1

## System Upgrade
apt-get -q -y -o "Dpkg::Options::=--force-confdef" -o "Dpkg::Options::=--force-confold" upgrade > /dev/null 2>&1

## Install packages needed
apt-get -f -q -y install curl unzip zip geoip-database apache2-dev autotools-dev libtool automake autoconf libgeoip-dev libxml2 libxml2-dev libxml2-utils libaprutil1 libaprutil1-dev libc6 libpcre3 libpcre3-dev libpcrecpp0 make gcc libssl-dev zlib1g zlib1g-dev lsb-base build-essential libssl-dev libgoogle-perftools-dev google-perftools > /dev/null 2>&1

## cleanup..
apt-get -o Dpkg::Options::='--force-confdef' -o Dpkg::Options::='--force-confold' -f -q -y autoremove > /dev/null 2>&1
apt-get autoclean > /dev/null 2>&1

## checking last versions of Nginx
d="/usr/local/src"
mkdir -p $d
changes=$(curl -s http://nginx.org/en/CHANGES | head -3 | grep "Changes with nginx" | awk '{ print $4 }')
ver=nginx-$changes.tar.gz
rm -rf $d/$ver
wget -q http://nginx.org/download/$ver -O $d/$ver
cd $d && tar xf $ver

## Change Nginx brand (server name on header)
sed -i 's/Server:\ nginx/Server:\ CloudMesh/g' $d/nginx-$changes/src/http/ngx_http_header_filter_module.c
sed -i 's/NGINX_VER\ CRLF/CRLF/g' $d/nginx-$changes/src/http/ngx_http_header_filter_module.c

## Download NAXSI WAF
wget -q https://github.com/nbs-system/naxsi/archive/master.zip -O naxsi.zip
unzip -o -qq naxsi.zip

## Download reCaptcha google
wget -q https://github.com/yaoweibin/nginx_http_recaptcha_module/archive/master.zip -O nginx_http_recaptcha_module.zip
unzip -o -qq nginx_http_recaptcha_module.zip

## Download testcookie
wget -q https://github.com/kyprizel/testcookie-nginx-module/archive/master.zip -O testcookie-nginx-module.zip
unzip -o -qq testcookie-nginx-module.zip

## Download fancy directory listing
wget -q https://github.com/aperezdc/ngx-fancyindex/archive/master.zip -O fancyindex.zip
unzip -o -qq fancyindex.zip

## Download eval
wget -q https://github.com/openresty/nginx-eval-module/archive/master.zip -O nginx_eval_module.zip
unzip -o -qq nginx_eval_module.zip

## Download  SecureCookie
wget -q https://github.com/yaoweibin/nginx_secure_cookie_module/archive/master.zip -O secure_cookie_module.zip
unzip -o -qq secure_cookie_module.zip

## Download upload-progress
wget -q https://github.com/masterzen/nginx-upload-progress-module/archive/master.zip -O upload-progress.zip
unzip -o -qq upload-progress.zip

## Download Enhanced Nginx Memached Module
wget -q https://github.com/bpaquet/ngx_http_enhanced_memcached_module/archive/master.zip -O enhanced_memcached_module.zip
unzip -o -qq enhanced_memcached_module.zip

## Download Google PageSpeed
NPS_VERSION=1.10.33.4
wget -q https://github.com/pagespeed/ngx_pagespeed/archive/release-${NPS_VERSION}-beta.zip
unzip -o -qq release-${NPS_VERSION}-beta.zip
cd ngx_pagespeed-release-${NPS_VERSION}-beta/
wget -q https://dl.google.com/dl/page-speed/psol/${NPS_VERSION}.tar.gz
tar -xzf ${NPS_VERSION}.tar.gz

cd -

## Download Headers More
wget -q https://github.com/openresty/headers-more-nginx-module/archive/master.zip -O headers-more-nginx-module.zip
unzip -o -qq headers-more-nginx-module.zip

## Download Adobe f4f format
wget -q https://github.com/inventos/f4f-hds/archive/master.zip -O f4f-hds.zip
unzip -o -qq f4f-hds.zip

## Download, compile and install mod_security
wget -q https://github.com/SpiderLabs/ModSecurity/archive/master.zip -O mod_security.zip
unzip -o -qq mod_security
cd ModSecurity-master/
./autogen.sh > /dev/null 2>&1
./configure --enable-standalone-module > /dev/null 2>&1
make install > /dev/null 2>&1

#rm -rf *.zip

cd $d/nginx-$changes/

## Compile options
./configure --add-module=../naxsi-master/naxsi_src --add-module=../ModSecurity-master/nginx/modsecurity --add-module=../ngx_pagespeed-release-${NPS_VERSION}-beta --add-module=../f4f-hds-master/ --add-module=../headers-more-nginx-module-master/ --add-module=../ngx_http_enhanced_memcached_module-master/ --add-module=../nginx-upload-progress-module-master/ --add-module=../nginx-eval-module-master --add-module=../nginx_secure_cookie_module-master --add-module=../nginx_http_recaptcha_module-master --add-module=../testcookie-nginx-module-master --add-module=../ngx-fancyindex-master --with-http_gzip_static_module --with-http_sub_module --with-http_addition_module --with-google_perftools_module --with-http_stub_status_module --with-http_ssl_module --with-http_secure_link_module --with-http_realip_module --with-ipv6 --with-http_addition_module --with-http_flv_module --with-http_mp4_module --with-http_geoip_module --with-threads --with-file-aio --with-http_gunzip_module --with-http_v2_module > /dev/null 2>&1

## install
make > /dev/null 2>&1
make install > /dev/null 2>&1

## Link binary file to system directory
rm -rf /usr/local/sbin/nginx
ln -s /usr/local/nginx/sbin/nginx /usr/local/sbin/

## add nginx user
adduser --system --no-create-home --disabled-login --disabled-password --group nginx --home /usr/local/nginx/ --quiet

## Startup script
wget -q https://raw.github.com/JasonGiedymin/nginx-init-ubuntu/master/nginx -O /etc/init.d/nginx
chmod +x /etc/init.d/nginx
/usr/sbin/update-rc.d -f nginx defaults

## Systemd service file
#cat >/lib/systemd/system/nginx.service <<EOL
#EOL

#sleep 5m

cd ; rm -rf $d/*

## Download and decompress GeoIP database from maxmind
rm -rf /usr/share/GeoIP/
mkdir -p /usr/share/GeoIP/
wget -q http://geolite.maxmind.com/download/geoip/database/GeoIPv6.dat.gz -O /usr/share/GeoIP/GeoIPv6.dat.gz
wget -q http://geolite.maxmind.com/download/geoip/database/GeoLiteCityv6-beta/GeoLiteCityv6.dat.gz -O /usr/share/GeoIP/GeoLiteCityv6.dat.gz
wget -q http://download.maxmind.com/download/geoip/database/asnum/GeoIPASNumv6.dat.gz -O /usr/share/GeoIP/GeoIPASNumv6.dat.gz
wget -q http://geolite.maxmind.com/download/geoip/database/GeoLiteCountry/GeoIP.dat.gz -O /usr/share/GeoIP/GeoIP.dat.gz
rm -rf /usr/share/GeoIP/GeoLiteCityv6.dat  /usr/share/GeoIP/GeoIPv6.dat /usr/share/GeoIP/GeoIPASNumv6.dat
gzip -d /usr/share/GeoIP/GeoLiteCityv6.dat.gz
gzip -d /usr/share/GeoIP/GeoIPv6.dat.gz
gzip -d /usr/share/GeoIP/GeoIPASNumv6.dat.gz
gzip -d /usr/share/GeoIP/GeoIP.dat.gz

## Create log&cache directory
mkdir -p /var/cache/nginx/proxycache /var/log/nginx/


